import React, { createRef, useMemo, useState } from 'react';
import { View, Modal, Animated, TouchableOpacity, StyleSheet, useWindowDimensions, Platform } from 'react-native';
import { TooltipMenuItem } from './TooltipMenuItem';
export const TooltipMenu = _ref => {
  let {
    children,
    items,
    style,
    overlayStyle,
    widthType,
    labelContainerStyle,
    labelStyle,
    modalButtonStyle,
    onRequestClose,
    trianglePosition
  } = _ref;
  const {
    width: windowWidth,
    height: windowHeight
  } = useWindowDimensions();
  const buttonRef = /*#__PURE__*/createRef();
  const [isModalOpen, setModalOpen] = useState(false);
  const [componentPosition, setComponentPosition] = useState();
  const opacity = useMemo(() => new Animated.Value(0), []);

  const toggleModal = () => {
    setModalOpen(!isModalOpen);
  };

  const openModal = () => {
    Animated.timing(opacity, {
      toValue: 1,
      duration: 300,
      useNativeDriver: true
    }).start(toggleModal);
  };

  const calculateItemWith = () => {
    switch (widthType) {
      case 'half':
        return windowWidth / 2;

      case 'full':
        return windowWidth;

      default:
        return componentPosition === null || componentPosition === void 0 ? void 0 : componentPosition.width;
    }
  };

  const hideModal = () => {
    Animated.timing(opacity, {
      toValue: 0,
      duration: 300,
      useNativeDriver: true
    }).start(toggleModal);
  };

  const handleClick = onClickItem => {
    const method = isModalOpen ? hideModal : openModal;
    method();
    onClickItem();
  };

  const calculatePosition = event => {
    const width = event.nativeEvent.layout.width;
    const height = event.nativeEvent.layout.height;

    if (!componentPosition && buttonRef.current) {
      buttonRef.current.measureInWindow((x, y) => {
        setComponentPosition({
          x,
          y,
          width,
          height
        });
      });
    }
  };

  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(View, {
    style: style,
    ref: buttonRef
  }, /*#__PURE__*/React.createElement(TouchableOpacity, {
    onPress: openModal,
    onLayout: event => calculatePosition(event)
  }, children)), /*#__PURE__*/React.createElement(Modal, {
    visible: isModalOpen,
    transparent: true,
    onRequestClose: onRequestClose
  }, /*#__PURE__*/React.createElement(View, {
    style: [styles.overlay, overlayStyle]
  }), /*#__PURE__*/React.createElement(TouchableOpacity, {
    activeOpacity: 1,
    style: [{
      flex: 1
    }, modalButtonStyle],
    onPress: hideModal
  }, /*#__PURE__*/React.createElement(View, {
    style: [styles.component, componentPosition && {
      top: componentPosition.y,
      left: componentPosition.x
    }]
  }, /*#__PURE__*/React.createElement(TouchableOpacity, {
    onPress: isModalOpen ? hideModal : openModal
  }, children)), /*#__PURE__*/React.createElement(Animated.View, {
    style: [{
      minWidth: calculateItemWith()
    }, styles.tooltipContainer, componentPosition && {
      bottom: windowHeight - componentPosition.y + Platform.select({
        default: 10,
        android: -15
      }),
      left: componentPosition.x
    }, {
      opacity
    }]
  }, items.map((item, index) => {
    const classes = [];

    if (labelContainerStyle) {
      classes.push(labelContainerStyle);
    }

    if (index !== items.length - 1) {
      classes.push(styles.tooltipMargin);
    }

    return /*#__PURE__*/React.createElement(TooltipMenuItem, {
      key: index,
      label: item.label,
      onPress: () => handleClick(item.onPress),
      containerStyle: classes,
      labelStyle: labelStyle,
      testID: item.testID
    });
  })), /*#__PURE__*/React.createElement(Animated.View, {
    style: [styles.triangle, {
      opacity
    }, componentPosition && {
      top: componentPosition.y - 10
    }, componentPosition && trianglePosition === 'center' && {
      left: componentPosition.x + componentPosition.width / 2 - 10
    }, componentPosition && trianglePosition === 'left' && {
      left: componentPosition.x + 20
    }, componentPosition && trianglePosition === 'right' && {
      left: componentPosition.x + componentPosition.width - 40
    }]
  }))));
};
TooltipMenu.defaultProps = {
  widthType: 'auto',
  onRequestClose: () => {},
  trianglePosition: 'center'
};
const styles = StyleSheet.create({
  overlay: { ...StyleSheet.absoluteFillObject,
    backgroundColor: 'rgba(0,0,0,0.5)'
  },
  tooltipMargin: {
    borderBottomWidth: 1,
    borderBottomColor: '#E1E1E1'
  },
  component: {
    position: 'absolute'
  },
  tooltipContainer: {
    backgroundColor: 'white',
    borderRadius: 10,
    position: 'absolute'
  },
  triangle: {
    position: 'absolute',
    width: 10,
    height: 10,
    backgroundColor: 'transparent',
    borderStyle: 'solid',
    borderTopWidth: 10,
    borderRightWidth: 10,
    borderBottomWidth: 0,
    borderLeftWidth: 10,
    borderTopColor: 'white',
    borderRightColor: 'transparent',
    borderBottomColor: 'transparent',
    borderLeftColor: 'transparent'
  }
});
//# sourceMappingURL=TooltipMenu.js.map